@ECHO OFF

SET scriptPath=%~dp0

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Checking for administrative permissions...
ECHO -------------------------------------------------------------------------------
OPENFILES >nul 2>nul
if %errorLevel% == 0 (
	ECHO.
	ECHO -------------------------------------------------------------------------------
	ECHO Success: Administrative permissions confirmed.
	ECHO -------------------------------------------------------------------------------
) else (
	CALL %scriptPath%_message.error.cmd "Failure: Current permissions inadequate."
	EXIT /B !errorlevel!
)


REM %~1 expands %1 removing any surrounding quotes (")
SET target_dir=%~1
SET package_name=%~2
SET service=%~3
SET description=%~4
SET main_app=%~5
SET domain=%~6
SET username=%~7
SET password=%~8
SET autostart=%~9

SET last_error=0

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Running: %~nx0
ECHO Path: %~dp0
ECHO Installing app: %service%
ECHO Description: %description%
ECHO -------------------------------------------------------------------------------

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Checking input parameters...
ECHO -------------------------------------------------------------------------------
IF "%target_dir%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: Final directory for the installation must be provided."
	GOTO :finish
)
IF "%package_name%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: Package name must be provided."
	GOTO :finish
)
IF "%service%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: Service name must be provided."
	GOTO :finish
)
ECHO -------------------------------------------------------------------------------
ECHO Input parameters ok:
ECHO   app root=%target_dir%
ECHO   package_name=%package_name%
ECHO   autostart=%WP_AUTOSTART%
ECHO   WP_COPY_UTIL=%WP_COPY_UTIL%
ECHO   WP_SVC_UTIL=%WP_SVC_UTIL%
ECHO -------------------------------------------------------------------------------


REM Path to this .cmd file
REM Example: c:\test\appname\content\tools\lib\
SET WP_THIS_FILE_DIR=%~dp0

REM This .cmd file is in 'lib' directory:
SET orig_lib_dir=%WP_THIS_FILE_DIR%

REM directory where the nupkg originally installed
REM Example:
REM   %cd%  : c:\test
REM   %~dp0 : c:\test\appname\content\tools\lib\
REM
REM SET orig_package_dir=%cd%\%package_name%
REM original 'content' subdirectory
REM SET orig_content_dir=%orig_package_dir%\content

REM Get directory path 2 levels up from current .cmd file
REM 1. Stores the current directory for use by the POPD command, then
REM changes to the specified directory.
pushd %orig_lib_dir%..\..
REM 2. Store the directory
SET orig_content_dir=%CD%
REM 3. Changes to the directory stored by the PUSHD command.
popd


REM now directories for a new location (inside target_dir)

REM 'content' subdirectory
REM SET content_dir=%target_dir%\content
SET content_dir=%target_dir%

REM Where the .exe are initially downloaded and stored
SET src_dir=%content_dir%\dist
REM Where .exe are intended to be
SET dst_dir=%content_dir%

REM directory where all .exe to be placed
SET exe_dir=%dst_dir%
REM directory of main app script (i.e. app,js)
SET app_dir=%content_dir%

IF EXIST "%target_dir%\" (
	ECHO.
	ECHO -------------------------------------------------------------------------------
	ECHO This app's root exists indicating it has been installed previously.
	ECHO Trying to uninstall the app...
	ECHO -------------------------------------------------------------------------------
	CALL "%orig_lib_dir%_uninstall.cmd" "%target_dir%" "%package_name%" "%service%"
	IF ERRORLEVEL 1 (
		CALL %scriptPath%_message.error.cmd "ERROR: Unable to uninstall the app."
		EXIT /B !errorlevel!
	)
	ECHO The app is uninstalled (service removed and root folder removed)
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Creating the app root folder...
ECHO -------------------------------------------------------------------------------
CALL md "%dst_dir%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR: failed while creating app root folder"
	EXIT /B !errorlevel!
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [1/8] Copy package content to app root...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_copy_all.cmd" "%orig_content_dir%" "%dst_dir%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR: Failed copying package content to app root"
	EXIT /B !errorlevel!
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [2/8] Copy binaries from dist to default location...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_copy_exe.cmd" "%src_dir%" "%dst_dir%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR: Failed copying binaries to app root"
	EXIT /B !errorlevel!
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [3/8] Running preinstall.js...
ECHO -------------------------------------------------------------------------------
ECHO ON
"%dst_dir%\node.exe" "%dst_dir%\tools\lib\preinstall.js"
@ECHO OFF
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR: preinstall.js failed"
	EXIT /B !errorlevel!
)


REM variable `install_mode` defines which utility is used to handle windows service
REM available values are: `winsw` || `nssm`
REM SET install_mode=winsw
REM IF "%install_mode%"=="winsw" GOTO :winsw
REM IF "%install_mode%"=="nssm" GOTO :nssm
IF /I "%WP_SVC_UTIL%"=="winsw" GOTO :winsw
IF /I "%WP_SVC_UTIL%"=="nssm" GOTO :nssm
CALL %scriptPath%_message.error.cmd "FATAL: WP_SVC_UTIL must be -winsw or -nssm"
SET last_error=100
GOTO :finish


:winsw
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [4/8] Creating service wrapper using winsw...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_winsw_create_wrapper.cmd" "%exe_dir%" "%app_dir%" "%service%" "%main_app%" "%description%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [5/8] Installing service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_winsw_svc_install.cmd" "%exe_dir%" "%app_dir%" "%service%" "%main_app%" "%description%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [6/8] Starting service...
ECHO -------------------------------------------------------------------------------
IF /I "%WP_AUTOSTART%"=="-Start" GOTO :winsw_start_svc
IF /I "%WP_AUTOSTART%"=="-NoStart" GOTO :winsw_skip_start
CALL %scriptPath%_message.error.cmd "FATAL: WP_AUTOSTART must be one of following: -Start or -NoStart"
SET last_error=100
GOTO :finish

:winsw_start_svc
CALL "%orig_lib_dir%install\_winsw_svc_start.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)
GOTO :winsw_start_finish

:winsw_skip_start
ECHO WP_AUTOSTART is set to -NoStart, skipping...

:winsw_start_finish
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [7/8] Checking status...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_winsw_svc_status.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)
GOTO :postinstall


:nssm
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [4/8] Installing the service using nssm...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_nssm_svc_install.cmd" "%exe_dir%" "%app_dir%" "%service%" "%main_app%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [5/8] Setting description for the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_nssm_svc_desc.cmd" "%exe_dir%" "%service%" "%description%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [6/8] Setting username/password for the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%install\_nssm_svc_user_pass.cmd" "%exe_dir%" "%service%" "%domain%" "%username%" "%password%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [7/8] Starting service...
ECHO -------------------------------------------------------------------------------

IF /I "%WP_AUTOSTART%"=="-Start" GOTO :nssm_start_svc
IF /I "%WP_AUTOSTART%"=="-NoStart" GOTO :nssm_skip_start
CALL %scriptPath%_message.error.cmd "FATAL: WP_AUTOSTART must be one of following: -Start or -NoStart"
SET last_error=100
GOTO :finish

:nssm_start_svc
CALL "%orig_lib_dir%install\_nssm_svc_start.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (EXIT /B %ERRORLEVEL%)
GOTO :nssm_start_finish

:nssm_skip_start
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO WP_AUTOSTART is set to -NoStart, skipping...
ECHO -------------------------------------------------------------------------------

:nssm_start_finish
GOTO :postinstall

:postinstall
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Service Controller check state of "%service%
ECHO -------------------------------------------------------------------------------
sc query "%service%"
IF NOT ERRORLEVEL 0 (
	CALL %scriptPath%_message.error.cmd "ERROR: The service does not exist"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0"  EXIT /B 1
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [8/8] Running postinstall.js...
ECHO -------------------------------------------------------------------------------
ECHO ON
"%dst_dir%\node.exe" "%dst_dir%\tools\lib\postinstall.js"
@ECHO OFF
IF ERRORLEVEL 1 (EXIT /B %errorlevel%)


:finish
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Done running: %~nx0
ECHO Path: %~dp0
IF NOT "%last_error%"=="0" (
	ECHO There were errors during the installation
	ECHO Please, check the logs
)
ECHO -------------------------------------------------------------------------------
EXIT /B %last_error%
