@ECHO OFF

SET scriptPath=%~dp0

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Checking for administrative permissions...
ECHO -------------------------------------------------------------------------------
OPENFILES >nul 2>nul
if %errorLevel% == 0 (
	ECHO.
	ECHO -------------------------------------------------------------------------------
	ECHO Success: Administrative permissions confirmed.
	ECHO -------------------------------------------------------------------------------
) else (
	CALL %scriptPath%_message.error.cmd "Failure: Current permissions inadequate."
	EXIT /B !errorlevel!
)


REM %~1 expands %1 removing any surrounding quotes (")
SET target_dir=%~1
SET package_name=%~2
SET service=%~3

SET last_error=0
SET break_on_error=1

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Running: %~nx0
ECHO Path: %~dp0
ECHO Uninstalling app: %service%
ECHO Description: %description%
ECHO -------------------------------------------------------------------------------

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Checking input parameters...
ECHO -------------------------------------------------------------------------------
IF "%target_dir%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: App root directory for the installation must be provided."
	GOTO :finish
)
IF "%package_name%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: Package name must be provided."
	GOTO :finish
)
IF "%service%"=="" (
	SET last_error=100
	CALL %scriptPath%_message.error.cmd "ERROR: Service name must be provided."
	GOTO :finish
)
ECHO -------------------------------------------------------------------------------
ECHO Input parameters ok:
ECHO   app root=%target_dir%
ECHO   package_name=%package_name%
ECHO   autostart=%WP_AUTOSTART%
ECHO -------------------------------------------------------------------------------


REM Path to this .cmd file
REM Example: c:\test\appname\content\tools\lib\
SET WP_THIS_FILE_DIR=%~dp0

REM This .cmd file is in 'lib' directory:
SET orig_lib_dir=%WP_THIS_FILE_DIR%

REM directory where the nupkg originally installed
REM Example:
REM   %cd%  : c:\test
REM   %~dp0 : c:\test\appname\content\tools\lib\_install.cmd
REM
REM SET orig_package_dir=%cd%\%package_name%
REM original 'content' subdirectory
REM SET orig_content_dir=%orig_package_dir%\content
REM SET orig_content_dir=%~dp0..\..\

REM Get directory path 2 levels up from current .cmd file
REM 1. Stores the current directory for use by the POPD command, then
REM changes to the specified directory.
pushd %orig_lib_dir%..\..
REM 2. Store the directory
SET orig_content_dir=%CD%
REM 3. Changes to the directory stored by the PUSHD command.
popd


REM now directories for a new location (inside target_dir)

REM 'content' subdirectory
REM SET content_dir=%target_dir%\content
SET content_dir=%target_dir%

REM Where the .exe are initially downloaded and stored
SET src_dir=%content_dir%\dist
REM Where .exe are intended to be
SET dst_dir=%content_dir%

REM directory where located all .exe
SET exe_dir=%dst_dir%
REM directory of main app script (i.e. app,js)
SET app_dir=%content_dir%


ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Checking if service "%service%" needs to be removed...
ECHO -------------------------------------------------------------------------------
sc query "%service%">NUL
IF ERRORLEVEL 1 (
	ECHO.
	ECHO -------------------------------------------------------------------------------
	ECHO The service does not exist.
	ECHO -------------------------------------------------------------------------------
	GOTO :deleteDir
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO The service exists; need to remove it.
ECHO -------------------------------------------------------------------------------

REM variable `WP_UNINSTALL_MODE` defines which utility is used to handle windows service
REM valid values are: `winsw` || `nssm`
SET WP_UNINSTALL_MODE=undefined
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Identifying which utility is used to handle windows service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_get_svc_mode.cmd" "%exe_dir%" "%service%"
IF "%WP_UNINSTALL_MODE%"=="winsw" GOTO :winsw
IF "%WP_UNINSTALL_MODE%"=="nssm" GOTO :nssm
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO WARN: WP_UNINSTALL_MODE nor winsw, nor nssm. Skipping service removal...
ECHO -------------------------------------------------------------------------------
GOTO :checkServiceGone

:winsw
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO winsw utility is used to handle windows service.
ECHO -------------------------------------------------------------------------------

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [1/4] Stopping the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_winsw_svc_stop.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR code: %errorlevel%"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0" GOTO :finish
)
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [2/4] Removing the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_winsw_svc_remove.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR code: %errorlevel%"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0" GOTO :finish
)
GOTO :checkServiceGone


:nssm
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO nssm utility is used to handle windows service...
ECHO -------------------------------------------------------------------------------

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [1/4] Stopping the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_nssm_svc_stop.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR code: %errorlevel%"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0" GOTO :finish
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [2/4] Removing the service...
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_nssm_svc_remove.cmd" "%exe_dir%" "%service%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR code: %errorlevel%"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0" GOTO :finish
)
GOTO :checkServiceGone

:checkServiceGone
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Pausing for 10 seconds before confirming the service has been removed...
ECHO -------------------------------------------------------------------------------
ping 127.0.0.1 -n 11 > nul

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Confirming that "%service%" is gone...
ECHO -------------------------------------------------------------------------------
sc query "%service%">NUL
IF NOT ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR: The service still exists"
	SET last_error=1
	IF NOT "%break_on_error%"=="0" GOTO :finish
) ELSE (
	ECHO The service is gone - service check failed as expected.
)

REM We need to remove 'code' directory inside app directory, not full app directory as
REM 'outside' _config directory may be located in app directory and must not be touched by install/uninstall.
REM Example:
REM Code directory:
REM   'C:\Program Files\apps\approot\code'
REM App directory:
REM   'C:\Program Files\apps\approot'
REM Outside config directory (must not be touched by install/uninstall):
REM   'C:\Program Files\apps\approot\_config'
REM Inside config directory (must be created/removed during install/uninstall):
REM   'C:\Program Files\apps\approot\code\_config'
:deleteDir
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [3/4] Recursively deleting app root ...
ECHO %target_dir%
ECHO -------------------------------------------------------------------------------
CALL "%orig_lib_dir%uninstall\_rmdir_sync.cmd" "%target_dir%"
IF ERRORLEVEL 1 (
	CALL %scriptPath%_message.error.cmd "ERROR code: %errorlevel%"
	SET last_error=%errorlevel%
	IF NOT "%break_on_error%"=="0" GOTO :finish
)

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO [4/4] Checking for stray code file in app root parent ...
ECHO -------------------------------------------------------------------------------
IF EXIST "%target_dir%" (
	ECHO Found stray code file, removing it ...
	CALL del "%dst_dir%" /s /q
	IF ERRORLEVEL 1 (
		CALL %scriptPath%_message.error.cmd "ERROR: %errorlevel%"
		SET last_error=%errorlevel%
		IF NOT "%break_on_error%"=="0" GOTO :finish
	)
)


:finish
ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Done running: %~nx0
ECHO Path: %~dp0
IF NOT "%last_error%"=="0" GOTO :exit_error
ECHO -------------------------------------------------------------------------------
EXIT /B 0

:exit_error
ECHO There were errors during the uninstallation
ECHO Please, check the logs
ECHO -------------------------------------------------------------------------------
EXIT /B %last_error%
