@ECHO OFF

SET dir_from=%~1
SET dir_to=%~2

SET prefix=[%~n0]

ECHO -------------------------------------------------------------------------------
ECHO %prefix%
ECHO %prefix% Running %0
ECHO %prefix% Copying the directory %dir_from% to %dir_to%
ECHO %prefix% Input:
ECHO %prefix%   WP_COPY_UTIL=%WP_COPY_UTIL%
ECHO %prefix%   dir_from=%dir_from%
ECHO %prefix%   dir_to=%dir_to%
ECHO -------------------------------------------------------------------------------

SET last_error=0

REM Choose which command to use: XCOPY or ROBOCOPY

GOTO :check_copy_util

REM Determine if ROBOCOPY is installed
REM Not used as the moment

REM Try to run ROBOCOPY hiding all output
ROBOCOPY >nul 2>nul
REM process errorlevel returned by robocopy
REM %errorlevel% 1 || 2 || 4  is not error for robocopy
SET /A ROBO_ERROR="%ERRORLEVEL% & 24"
IF "%ROBO_ERROR%"=="16" GOTO :ROBOCOPY_INSTALLED
ECHO -------------------------------------------------------------------------------
ECHO %prefix% ROBOCOPY command not found, fall back to XCOPY
SET WP_COPY_UTIL=xcopy
:robocopy_installed
ECHO %prefix% ROBOCOPY command found
SET WP_COPY_UTIL=robocopy
ECHO -------------------------------------------------------------------------------

:check_copy_util
IF /I "%WP_COPY_UTIL%"=="robocopy" GOTO :ROBOCOPY_START
IF /I "%WP_COPY_UTIL%"=="xcopy" GOTO :XCOPY_START
ECHO -------------------------------------------------------------------------------
ECHO WP_COPY_UTIL must be robocopy or xcopy
ECHO -------------------------------------------------------------------------------
SET last_error=1
GOTO :finish


:ROBOCOPY_START
ECHO -------------------------------------------------------------------------------
ECHO %prefix% Using ROBOCOPY command
ECHO -------------------------------------------------------------------------------
ECHO ON
ROBOCOPY "%dir_from%" "%dir_to%" /E /LOG:robocopy.log
@ECHO OFF
REM
REM process errorlevel returned by robocopy
REM %errorlevel% 1 || 2 || 4  is not error for robocopy
REM SET /A ROBO_ERROR="%ERRORLEVEL% & 24"
ECHO -------------------------------------------------------------------------------
ECHO %prefix% INFO: ROBOCOPY returned code: %ERRORLEVEL% (ok if 1)
IF %ERRORLEVEL% EQU 1 GOTO :finish
ECHO %prefix% ERROR: %ERRORLEVEL% greater than 8
ECHO -------------------------------------------------------------------------------
SET last_error=%ERRORLEVEL%
GOTO :finish


:XCOPY_START
REM XCOPY does not allow to copy files with long names
REM ECHO ROBOCOPY command not found, fall back to XCOPY
ECHO -------------------------------------------------------------------------------
ECHO %prefix% Using XCOPY command
ECHO -------------------------------------------------------------------------------
ECHO ON
XCOPY /E /I /Q /Y /J "%dir_from%" "%dir_to%"
@ECHO OFF
ECHO -------------------------------------------------------------------------------
ECHO %prefix% XCOPY exit code: %ERRORLEVEL%
ECHO -------------------------------------------------------------------------------
IF "%ERRORLEVEL%"=="0" GOTO :finish
SET last_error=%ERRORLEVEL%
GOTO :finish

:finish
ECHO -------------------------------------------------------------------------------
ECHO %prefix% Output:
ECHO %prefix%   exit code=%last_error%
ECHO.
IF "%last_error%"=="0" (
  ECHO %prefix% SUCCESS
) ELSE (
  ECHO %prefix% ERRORS FOUND
)
ECHO %prefix%
ECHO -------------------------------------------------------------------------------
EXIT /B %last_error%
