@ECHO OFF
SET repo_name=%~1
SET repo_url=%~2
SET repo_branch=%~3
SET repo_user=%~4
SET repo_password=%~5

SET prefix=[%~n0]

ECHO %prefix%
ECHO %prefix% %DATE% %TIME%
ECHO %prefix% Running %0
ECHO %prefix% Clone/pull git repository %repo_name%
ECHO %prefix%
ECHO %prefix% Input:
ECHO %prefix%   repo_name=%repo_name%
ECHO %prefix%   repo_url=%repo_url%
ECHO %prefix%   repo_branch=%repo_branch%
ECHO %prefix%   repo_user=%repo_user%
ECHO %prefix%   repo_password=%repo_password%

SET last_error=0

SET full_repo_url=https://%repo_user%:%repo_password%@%repo_url%

REM Simplest way to check if subdirectory of current directory is a root of Git repository
IF EXIST %repo_name%\.git\ (
  ECHO %prefix% INFO: .git subdirectory found, subdirectory %repo_name% is a root of Git repository
  GOTO :do_pull
) ELSE (
  ECHO %prefix% INFO: .git subdirectory does not exist, subdirectory %repo_name% is not a root of existing Git repository
  GOTO :do_clone
)
REM Alternative way to check if we are _inside_ Git repository
REM git rev-parse --git-dir >nul 2>nul
REM IF %ERRORLEVEL%==0 GOTO :do_pull
REM GOTO :do_clone

:do_clone
ECHO ON
git clone %full_repo_url%
@ECHO OFF
IF "%ERRORLEVEL%"=="0" GOTO :do_checkout
ECHO %prefix% ERROR: git clone returned code %ERRORLEVEL%
SET last_error=%ERRORLEVEL%
GOTO :finish

:do_pull
PUSHD %repo_name%
ECHO ON
git pull
@ECHO OFF
POPD
IF "%ERRORLEVEL%"=="0" GOTO :do_checkout
ECHO %prefix% ERROR: git pull returned code %ERRORLEVEL%
SET last_error=%ERRORLEVEL%
GOTO :finish

:do_checkout
PUSHD %repo_name%
ECHO ON
git checkout %repo_branch%
@ECHO OFF
POPD
IF "%ERRORLEVEL%"=="0" GOTO :finish
ECHO %prefix% ERROR: git checkout returned code %ERRORLEVEL%
SET last_error=%ERRORLEVEL%
GOTO finish

:finish
ECHO %prefix%
ECHO %prefix% %DATE% %TIME%
ECHO %prefix% Output:
ECHO %prefix%   EXIT /B %last_error%
ECHO.
IF "%last_error%"=="0" (
  ECHO %prefix% SUCCESS
) ELSE (
  ECHO %prefix% ERRORS FOUND
)
ECHO %prefix%
EXIT /B %last_error%

