@ECHO OFF

REM Current .cmd file directory
REM Must be read before using 'SHIFT' command
SET tools_dir=%~dp0

REM Read switches from command line

REM Set default values for the switches
SET WP_AUTOSTART=-Start
SET WP_SVC_UTIL=WINSW
SET WP_COPY_UTIL=ROBOCOPY
REM Read switches
:read_switches
REM Must be no space between variable value and '&' in lines below:
IF /I "%~1"=="-NoStart"  SET WP_AUTOSTART=%~1& GOTO read_switches_next
IF /I "%~1"=="-Start"    SET WP_AUTOSTART=%~1& GOTO read_switches_next
IF /I "%~1"=="-winsw"    SET WP_SVC_UTIL=winsw& GOTO read_switches_next
IF /I "%~1"=="-nssm"     SET WP_SVC_UTIL=nssm& GOTO read_switches_next
IF /I "%~1"=="-robocopy" SET WP_COPY_UTIL=robocopy& GOTO read_switches_next
IF /I "%~1"=="-xcopy"    SET WP_COPY_UTIL=xcopy& GOTO read_switches_next
GOTO read_switches_finish
:read_switches_next
SHIFT
GOTO read_switches
:read_switches_finish


REM Read other arguments from command line

SET domain=%~1
SET account=%~2
SET password=%~3

REM Usage example:
REM C:\test>appname\content\tools\install.cmd ".\test" "test"

REM parameters for _install.cmd:
REM
REM <config.install.target> <nupkg_package_name> <win_svc_name> <app_start_script> <username> <password>
REM
REM <config.install.target> - 1 - mandatory, target directory for the installation after unpacking the nupkg
REM <nupkg_package_name>    - 2 - mandatory, package name to install, also determines path to all its files after install
REM <win_svc_name>          - 3 - mandatory, windows service name to create, install, configure and start
REM <win_svc_description>   - 4 - mandatory, windows service description
REM <app_start_script>      - 5 - mandatory, main application .js script
REM <domain>                - 6 - optional, user domain for the service
REM <account>               - 7 - optional, user name for the service
REM <password>              - 8 - optional, user password for the service
REM
REM Environment variables used to pass data to _install.cmd:
REM WP_AUTOSTART            -  - Auto start the service after installation (-Start || -NoStart)
REM WP_SVC_UTIL             -  - Utility used to install the service (WINSW || NSSM)
REM WP_COPY_UTIL            -  - Utility used to copy the files (ROBOCOPY || XCOPY)
REM
REM Passing parameters to lib\_install.cmd using command line arguments considering deprecated,
REM use environment variables instead, prefixed with 'WP_'
REM Example: WP_AUTOSTART

ECHO.
ECHO *******************************************************************************
ECHO Starting installation of <%= config.nuget.name %>
ECHO *******************************************************************************

ECHO.
ECHO -------------------------------------------------------------------------------
ECHO Determining processor architecture...
IF /I "%PROCESSOR_ARCHITECTURE%"=="x86" GOTO 32
IF /I "%PROCESSOR_ARCHITECTURE%"=="AMD64" GOTO 64
ECHO ERROR: Unable to detect 32/64 architecture, exiting...
ECHO -------------------------------------------------------------------------------
EXIT /B 1

:32
ECHO 32-bit architecture detected
ECHO Install target is:
ECHO <%= config.install.target.processorArchitecture32bit %>
ECHO -------------------------------------------------------------------------------
CALL "%tools_dir%lib\_install.cmd" ^
	"<%= config.install.target.processorArchitecture32bit %>" ^
	"<%= config.nuget.name %>" ^
	"<%= config.windowsService.name %>" ^
	"<%= config.description || appPkg.description %>" ^
	"<%= appPkg.main %>" ^
	"%domain%" ^
	"%account%" ^
	"%password%"
GOTO done

:64
ECHO 64-bit architecture detected
ECHO Install target is:
ECHO <%= config.install.target.processorArchitecture64bit %>
ECHO -------------------------------------------------------------------------------
CALL "%tools_dir%lib\_install.cmd" ^
	"<%= config.install.target.processorArchitecture64bit %>" ^
	"<%= config.nuget.name %>" ^
	"<%= config.windowsService.name %>" ^
	"<%= config.description || appPkg.description %>" ^
	"<%= appPkg.main %>" ^
	"%domain%" ^
	"%account%" ^
	"%password%"
GOTO done

:done
ECHO.
ECHO *******************************************************************************
IF ERRORLEVEL 1 (
	ECHO ERROR: installation of <%= config.nuget.name %>
) ELSE (
	ECHO Completed installation of <%= config.nuget.name %>
)
ECHO *******************************************************************************
EXIT /B %errorlevel%
