'use strict';

var request = require('request');
var semver = require('semver');

function searchOptions(artifactName, repo) {
	switch (repo.type) {
		case "artifactory":
			return {
				method: 'GET',
				url: repo.url+'/artifactory/api/search/artifact?name='+artifactName,
				strictSSL: false,
				headers:{
					'Content-Type': 'text/plain'
				}
			};
		break;
		default:
			return null;
	}
}

function getNextPatchLevel(artifactName, version, repo, cb) {
	if (semver.valid(version)) {
		version = semver.major(version)+"."+semver.minor(version);
	} else {
		if (!semver.valid(version+".0")) {
			return cb(new Error("version not valid: "+version));
		}
	}
	
	var options = searchOptions(artifactName, repo);
	if (!options) {
		return cb(new Error("unknown artifact repo type: "+repo.type));
	}

	request(options, function (error, httpResponse, body) {

		if (error) {
			return cb(error);
		}

		// statusCode not OK
		if (httpResponse.statusCode < 200 || httpResponse.statusCode >= 300) {
			return cb(new Error("bad response: "+httpResponse.statusCode));
		}

		var responseBody = body.trim();

		// Empty response body
		if (!responseBody) {
			return cb(new Error("empty response"));
		}

		// Convert to JSON and check if it is valid
		var responseJson = null;
		try {
			responseJson = JSON.parse(responseBody);
		} catch (error) {
			return cb(new Error("JSON response parse error"));
		}

		//console.log(responseJson);
		var pattern = artifactName.replace(/\./, '\\.')+'\\.([\\d\\.]+)\\..+';
		var versions = responseJson.results.map(function(result) {
			return new RegExp(pattern, 'g').exec(result.uri)[1];
		});
		versions.filter(function(version) {
			return semver.valid(version);
		});
		
		var current = semver.maxSatisfying(versions, version+".x");
		if (current) {
			cb(null, semver.inc(current, 'patch'));
		} else {
			cb(null, version+".0");
		}
	});
}

module.exports = {
	getNextPatchLevel: getNextPatchLevel
};
