var npmview = require('npmview');
var semver = require('semver');
var npmUtils = require('npm-utils');
var fs = require('fs');
var git = require('git-state');

console.log('Build started.');

console.log('Checking that git repo is clean.');
git.check('./', function(e, result) {
	if (e) {
		console.error(e);
		console.error('Unable to check git repo state.');
		console.error('Build failed.');
		return;
	}
	if (result.dirty + result.untracked > 0) {
		console.error('git repo has issues.');
		console.log(result);
		console.error('Build failed.');
		return;
	}

	npmview('window-pain', function(err, version, moduleInfo) {
		if (err) {
			console.error(err);
			console.error('Publish failed.');
			return;
		}
	
		console.log('Current version on npmjs.com is '+version);
		var nextVersion = semver.inc(version, 'patch');
	
		var pkg = require('../package.json');
		var sourceVersion = pkg.version;
		pkg.version = nextVersion;
	
		console.log('Updating package to next version: '+pkg.version);
		fs.writeFile('./package.json', JSON.stringify(pkg, null, "\t"), {encoding : 'utf8'}, function(err) {
			if (err) {
				console.error(err);
				console.error('Build failed.');
				return;
			}
			console.log('Attempting to publish '+moduleInfo.name+'@'+nextVersion);
			npmUtils.publish()
				.then(function() {
					console.log('Restoring package.json.');
					pkg.version = sourceVersion;
					fs.writeFile('./package.json', JSON.stringify(pkg, null, "\t"), {encoding : 'utf8'}, function(err) {
						if (err) {
							console.error(err);
							console.error('Restoring package.json failed.');
							return;
						}
						console.log('Build completed.');
					});
				})
				.catch(function(e){
					console.log('Build failed.');
					console.log('Restoring package.json.');
					pkg.version = sourceVersion;
					fs.writeFile('./package.json', JSON.stringify(pkg, null, "\t"), {encoding : 'utf8'}, function(err) {
						if (err) {
							console.error(err);
							console.error('Restoring package.json failed.');
							return;
						}
					});		
				})
			;
		});
	});
});


